#*******************************************************************************************************************************
#
# This file contains the Vivado Tcl commands to create a Model 37X-YYY SigFPGA project.
# Note:	37X-YYY is the specific part number of the product you ordered from Red Rapids (e.g. Model 377-403).
#
# Instructions to execute from Vivado GUI mode:
# Enter the following three commands from the Vivado Tcl console:
#   set argv [ list 37X-YYY ]
#	set argc 1
#	source project.tcl
#
# Instructions to execute from Vivado batch mode:
# Enter the following command from an OS command shell:
#   vivado -mode batch -notrace -nojournal -nolog -source project.tcl -tclargs 37X-YYY
# Note: The "vivado" executable must be in your search path, otherwise provide the complete pathname.
#
#*******************************************************************************************************************************

# Break the part number (XXX-YYY) into a model number (XXX) and build option code (YYY).
set modelnumber [string range [lindex $argv 0] 0 2]
set buildoption [string range [lindex $argv 0] 4 6]

# Set variables based on the part number.
switch -exact $modelnumber {
	371 { set frequency "250000000";
		switch -exact $buildoption {
		001 { set physical "XMC"; set device "xc7k410tfbg676-2"; set sram "FF" }
		002 { set physical "XMC"; set device "xc7k410tfbg676-2"; set sram "NO" }
		100 { set physical "XMC"; set device "xc7k160tfbg676-2"; set sram "NO" }
		400 { set physical "PCI"; set device "xc7k410tfbg676-2"; set sram "SS" }
		default { puts "ERROR: Build option $buildoption is not available for model number $modelnumber"; exit }
		}
	}
	372 { set frequency "310000000";
		switch -exact $buildoption {
		002 { set physical "XMC"; set device "xc7k325tfbg676-2"; set sram "SS" }
		101 { set physical "XMC"; set device "xc7k325tfbg676-3"; set sram "NO" }
		500 { set physical "XMC"; set device "xc7k325tfbg676-2"; set sram "NO" }
		501 { set physical "XMC"; set device "xc7k325tfbg676-2"; set sram "NO" }
		default { puts "ERROR: Build option $buildoption is not available for model number $modelnumber"; exit }
		}
	}
	373 { set frequency "310000000";
		switch -exact $buildoption {
		001 { set physical "XMC"; set device "xc7k160tfbg676-2"; set sram "NO" }
		100 { set physical "XMC"; set device "xc7k160tfbg676-2"; set sram "NO" }
		101 { set physical "XMC"; set device "xc7k160tfbg676-2"; set sram "NO" }
		default { puts "ERROR: Build option $buildoption is not available for model number $modelnumber"; exit }
		}
	}
	376 { set frequency "400000000";
		switch -exact $buildoption {
		001 { set physical "XMC"; set device "xc7k410tfbg676-2"; set sram "FF" }
		003 { set physical "XMC"; set device "xc7k410tfbg676-2"; set sram "NO" }
		101 { set physical "XMC"; set device "xc7k410tfbg676-2"; set sram "FF" }
		default { puts "ERROR: Build option $buildoption is not available for model number $modelnumber"; exit }
		}
	}
	377 { set frequency "250000000";
		switch -exact $buildoption {
		403 { set physical "PCI"; set device "xc7k160tfbg676-2"; set sram "FF" }
		405 { set physical "PCI"; set device "xc7k410tfbg676-2"; set sram "FF" }
		default { puts "ERROR: Build option $buildoption is not available for model number $modelnumber"; exit }
		}
	}
	378 { set frequency "125000000";
		switch -exact $buildoption {
		001 { set physical "XMC"; set device "xc7k160tfbg676-2"; set sram "NO" }
		default { puts "ERROR: Build option $buildoption is not available for model number $modelnumber"; exit }
		}
	}
	default { puts "ERROR: Model number $modelnumber is invalid"; exit }
}

# Print the results of the part number decoder.
puts "Model Number: $modelnumber"
puts "Build Option: $buildoption"
puts "Physical: $physical"
puts "Device: $device"
puts "SRAM: $sram"


# Create a Managed IP project.
create_project managed_ip_project ../ip_sources/vivado/$device/managed_ip_project -part $device -ip -force

# Set project properties.
set obj [get_projects managed_ip_project]
set_property "simulator_language" "Mixed" $obj
set_property "target_language" "VHDL" $obj

# Set 'sources_1' fileset object.
set obj [get_filesets sources_1]
set files [list \
 "[file normalize "./../ip_sources/vivado/$device/dma_on_demand_memory_v1/dma_on_demand_memory.xci"]"\
 "[file normalize "./../ip_sources/vivado/$device/fifo_channel_1kx64_v1/fifo_channel_1kx64.xci"]"\
 "[file normalize "./../ip_sources/vivado/$device/fifo_dma_1kx64_v1/fifo_dma_1kx64.xci"]"\
 "[file normalize "./../ip_sources/vivado/$device/fifo_flash_512x32_v1/fifo_flash_512x32.xci"]"\
 "[file normalize "./../ip_sources/vivado/$device/pcie_ep_x8_gen2_axi128_250mhz_v2/pcie_endpoint.xci"]"\
]
# Select part number specific sourc files.
if {$modelnumber == "378"} {
  set files [concat $files \
   "[file normalize "./../ip_sources/vivado/$device/dma_ext_memory_v0/dma_ext_memory.xci"]"\
   "[file normalize "./../ip_sources/vivado/$device/fifo_16x64_v0/fifo_16x64.xci"]"\
  ]
}
# Include SRAM source files on models equipped with memory.
if {$sram != "NO"} {
  set files [concat $files \
   "[file normalize "./../ip_sources/vivado/$device/fifo_qdr_512x36_v0/fifo_qdr_512x36.xci"]"\
  ]
}
add_files -norecurse -fileset $obj $files

# Create a list of IP cores that need to have output products generated.
set ip {}

generate_target all [get_files  ./../ip_sources/vivado/$device/dma_on_demand_memory_v1/dma_on_demand_memory.xci]
export_ip_user_files -of_objects [get_files ./../ip_sources/vivado/$device/dma_on_demand_memory_v1/dma_on_demand_memory.xci] -no_script -force -quiet
set ip [concat $ip [create_ip_run [get_files -of_objects [get_fileset sources_1] ./../ip_sources/vivado/$device/dma_on_demand_memory_v1/dma_on_demand_memory.xci]]]

generate_target all [get_files  ./../ip_sources/vivado/$device/fifo_channel_1kx64_v1/fifo_channel_1kx64.xci]
export_ip_user_files -of_objects [get_files ./../ip_sources/vivado/$device/fifo_channel_1kx64_v1/fifo_channel_1kx64.xci] -no_script -force -quiet
set ip [concat $ip [create_ip_run [get_files -of_objects [get_fileset sources_1] ./../ip_sources/vivado/$device/fifo_channel_1kx64_v1/fifo_channel_1kx64.xci]]]
if {$frequency > 250000000} {
  set_property  CONFIG.READ_CLK.FREQ_HZ $frequency [get_ips  fifo_channel_1kx64]
  set_property CONFIG.WRITE_CLK.FREQ_HZ $frequency [get_ips fifo_channel_1kx64]
} else {
  set_property  CONFIG.READ_CLK.FREQ_HZ 250000000 [get_ips  fifo_channel_1kx64]
  set_property CONFIG.WRITE_CLK.FREQ_HZ 250000000 [get_ips fifo_channel_1kx64]
}

generate_target all [get_files  ./../ip_sources/vivado/$device/fifo_dma_1kx64_v1/fifo_dma_1kx64.xci]
export_ip_user_files -of_objects [get_files ./../ip_sources/vivado/$device/fifo_dma_1kx64_v1/fifo_dma_1kx64.xci] -no_script -force -quiet
set ip [concat $ip [create_ip_run [get_files -of_objects [get_fileset sources_1] ./../ip_sources/vivado/$device/fifo_dma_1kx64_v1/fifo_dma_1kx64.xci]]]
set_property CONFIG.CORE_CLK.FREQ_HZ 250000000 [get_ips fifo_dma_1kx64]

generate_target all [get_files  ./../ip_sources/vivado/$device/fifo_flash_512x32_v1/fifo_flash_512x32.xci]
export_ip_user_files -of_objects [get_files ./../ip_sources/vivado/$device/fifo_flash_512x32_v1/fifo_flash_512x32.xci] -no_script -force -quiet
set ip [concat $ip [create_ip_run [get_files -of_objects [get_fileset sources_1] ./../ip_sources/vivado/$device/fifo_flash_512x32_v1/fifo_flash_512x32.xci]]]
set_property  CONFIG.MASTER_ACLK.FREQ_HZ 250000000 [get_ips fifo_flash_512x32]
set_property  CONFIG.SLAVE_ACLK.FREQ_HZ 250000000 [get_ips fifo_flash_512x32]

generate_target all [get_files  ./../ip_sources/vivado/$device/pcie_ep_x8_gen2_axi128_250mhz_v2/pcie_endpoint.xci]
export_ip_user_files -of_objects [get_files ./../ip_sources/vivado/$device/pcie_ep_x8_gen2_axi128_250mhz_v2/pcie_endpoint.xci] -no_script -force -quiet
set ip [concat $ip [create_ip_run [get_files -of_objects [get_fileset sources_1] ./../ip_sources/vivado/$device/pcie_ep_x8_gen2_axi128_250mhz_v2/pcie_endpoint.xci]]]

if {$modelnumber == "378"} {
  generate_target all [get_files  ./../ip_sources/vivado/$device/dma_ext_memory_v0/dma_ext_memory.xci]
  export_ip_user_files -of_objects [get_files ./../ip_sources/vivado/$device/dma_ext_memory_v0/dma_ext_memory.xci] -no_script -force -quiet
  set ip [concat $ip [create_ip_run [get_files -of_objects [get_fileset sources_1] ./../ip_sources/vivado/$device/dma_ext_memory_v0/dma_ext_memory.xci]]]

  generate_target all [get_files  ./../ip_sources/vivado/$device/fifo_16x64_v0/fifo_16x64.xci]
  export_ip_user_files -of_objects [get_files ./../ip_sources/vivado/$device/fifo_16x64_v0/fifo_16x64.xci] -no_script -force -quiet
  set ip [concat $ip [create_ip_run [get_files -of_objects [get_fileset sources_1] ./../ip_sources/vivado/$device/fifo_16x64_v0/fifo_16x64.xci]]]
  set_property CONFIG.READ_CLK.FREQ_HZ $frequency [get_ips fifo_16x64]
  set_property CONFIG.WRITE_CLK.FREQ_HZ $frequency [get_ips fifo_16x64]
}

if {$sram != "NO"} {
  generate_target all [get_files  ./../ip_sources/vivado/$device/fifo_qdr_512x36_v0/fifo_qdr_512x36.xci]
  export_ip_user_files -of_objects [get_files ./../ip_sources/vivado/$device/fifo_qdr_512x36_v0/fifo_qdr_512x36.xci] -no_script -force -quiet
  set ip [concat $ip [create_ip_run [get_files -of_objects [get_fileset sources_1] ./../ip_sources/vivado/$device/fifo_qdr_512x36_v0/fifo_qdr_512x36.xci]]]
  set_property CONFIG.READ_CLK.FREQ_HZ 500000000 [get_ips fifo_qdr_512x36]
  set_property CONFIG.WRITE_CLK.FREQ_HZ 500000000 [get_ips fifo_qdr_512x36]
}

# Launch synthesis on all cores that need output products.
if {[llength $ip] != 0} {
  launch_run -jobs 12 $ip
  foreach ip_synth $ip {
    if {[lsearch -exact $ip $ip_synth] != -1} {wait_on_run $ip_synth}
  }
}

# Close the Managed IP project.
close_project

# Create reference design project.
create_project SigFPGA_Top ./Model_$modelnumber-$buildoption -part $device -force

# Set the directory path for the new project.
set proj_dir [get_property directory [current_project]]

# Set project properties.
set obj [get_projects SigFPGA_Top]
set_property "default_lib" "xil_defaultlib" $obj
set_property "part" $device $obj
set_property "sim.ip.auto_export_scripts" "1" $obj
set_property "simulator_language" "Mixed" $obj
set_property "target_language" "VHDL" $obj
set_property "xpm_libraries" "XPM_CDC XPM_MEMORY" $obj

# Create 'sources_1' fileset (if not found).
if {[string equal [get_filesets -quiet sources_1] ""]} {
  create_fileset -srcset sources_1
}

# Set 'sources_1' fileset object.
set obj [get_filesets sources_1]
# Include source files that are common to all model numbers.
set files [list \
 "[file normalize "./../../vhdl_models/bar0_decoder_v8.vhd"]"\
 "[file normalize "./../../vhdl_models/bar2_channel_v2.vhd"]"\
 "[file normalize "./../../vhdl_models/bar2_common_v7.vhd"]"\
 "[file normalize "./../../vhdl_models/bar2_decoder_v5.vhd"]"\
 "[file normalize "./../../vhdl_models/bpi_flash_v3.vhd"]"\
 "[file normalize "./../../vhdl_models/clock_frequency_v0.vhd"]"\
 "[file normalize "./../../vhdl_models/clock_lock_v6.vhd"]"\
 "[file normalize "./../../vhdl_models/clock_route_v4.vhd"]"\
 "[file normalize "./../../vhdl_models/dac_interface_v2.vhd"]"\
 "[file normalize "./../../vhdl_models/dma_on_demand_v2.vhd"]"\
 "[file normalize "./../../vhdl_models/dma_package_v1.vhd"]"\
 "[file normalize "./../../vhdl_models/dma_read_fifo_v1.vhd"]"\
 "[file normalize "./../../vhdl_models/dma_write_fifo_v2.vhd"]"\
 "[file normalize "./../../vhdl_models/interrupt_controller_v6.vhd"]"\
 "[file normalize "./../../vhdl_models/interrupt_timer_v1.vhd"]"\
 "[file normalize "./../../vhdl_models/pci_express_v3.vhd"]"\
 "[file normalize "./../../vhdl_models/pci_extended_dma_v0.vhd"]"\
 "[file normalize "./../../vhdl_models/pci_performance_v6.vhd"]"\
 "[file normalize "./../../vhdl_models/pci_transaction_decoder_v1.vhd"]"\
 "[file normalize "./../../vhdl_models/rx_datapath_v4.vhd"]"\
 "[file normalize "./../../vhdl_models/rx_flow_control_v3.vhd"]"\
 "[file normalize "./../../vhdl_models/serial_bus_v4.vhd"]"\
 "[file normalize "./../../vhdl_models/sync_enable_v0.vhd"]"\
 "[file normalize "./../../vhdl_models/sync_reset_v0.vhd"]"\
 "[file normalize "./../../vhdl_models/synchronizer_v1.vhd"]"\
 "[file normalize "./../../vhdl_models/transceiver_v11.vhd"]"\
 "[file normalize "./../../vhdl_models/transceiver_package_v5.vhd"]"\
 "[file normalize "./../../vhdl_models/tx_datapath_v3.vhd"]"\
 "[file normalize "./../../vhdl_models/tx_flow_control_v2.vhd"]"\
 "[file normalize "./../top_sources/M$modelnumber/$sram\_SRAM/model_specific_package_v10.vhd"]"\
 "[file normalize "./../top_sources/M$modelnumber/$sram\_SRAM/SigFPGA_top_v8.vhd"]"\
 "[file normalize "./../ip_sources/plda/ezdma2_axi128_250mhz_v0/plda_ezdma2.edn"]"\
 "[file normalize "./../ip_sources/vivado/$device/dma_on_demand_memory_v1/dma_on_demand_memory.xci"]"\
 "[file normalize "./../ip_sources/vivado/$device/fifo_channel_1kx64_v1/fifo_channel_1kx64.xci"]"\
 "[file normalize "./../ip_sources/vivado/$device/fifo_dma_1kx64_v1/fifo_dma_1kx64.xci"]"\
 "[file normalize "./../ip_sources/vivado/$device/fifo_flash_512x32_v1/fifo_flash_512x32.xci"]"\
 "[file normalize "./../ip_sources/vivado/$device/pcie_ep_x8_gen2_axi128_250mhz_v2/pcie_endpoint.xci"]"\
]
# Select part number specific sourc files.
if {$modelnumber == "378"} {
  set files [concat $files \
   "[file normalize "./../../vhdl_models/adc_serial_v0.vhd"]"\
   "[file normalize "./../../vhdl_models/frame_lock_v0.vhd"]"\
   "[file normalize "./../ip_sources/vivado/$device/fifo_16x64_v0/fifo_16x64.xci"]"\
   "[file normalize "./../ip_sources/vivado/$device/dma_ext_memory_v0/dma_ext_memory.xci"]"\
  ]
} else {
  set files [concat $files \
   "[file normalize "./../../vhdl_models/adc_interface_v6.vhd"]"\
  ]
}
# Include SRAM source files on models equipped with memory.
if {$sram != "NO"} {
  set files [concat $files \
   "[file normalize "./../../vhdl_models/qdr_dll_v2.vhd"]"\
   "[file normalize "./../../vhdl_models/qdr_sram_v5.vhd"]"\
   "[file normalize "./../../vhdl_models/qdr_demo_v2.vhd"]"\
   "[file normalize "./../ip_sources/vivado/$device/fifo_qdr_512x36_v0/fifo_qdr_512x36.xci"]"\
  ]
}
add_files -norecurse -fileset $obj $files

# Create 'constrs_1' fileset (if not found)
if {[string equal [get_filesets -quiet constrs_1] ""]} {
  create_fileset -constrset constrs_1
}

# Set 'constrs_1' fileset object
set obj [get_filesets constrs_1]
# Include constraint files that are common to all model numbers.
set files [list \
 "[file normalize "./../constraints/target.xdc"]"\
 "[file normalize "./../top_sources/M$modelnumber/model_specific_constraints_v1.xdc"]"\
 "[file normalize "./../constraints/SigFPGA_$physical\_v6.xdc"]"\
]
# Include SRAM constraint files on models equipped with memory.
if {$sram == "FF"} {
  set files [concat $files \
   "[file normalize "./../constraints/dual_qdr_ff_sram_v3.xdc"]"\
  ]
}
if {$sram == "SS"} {
  set files [concat $files \
   "[file normalize "./../constraints/dual_qdr_ss_sram_v3.xdc"]"\
  ]
}
add_files -norecurse -fileset $obj $files

# Set 'constrs_1' fileset properties
set obj [get_filesets constrs_1]
set_property "target_constrs_file" "[file normalize "./../constraints/target.xdc"]" $obj

# Suppress the "File Management" warnings produced for IP blocks during implementation.
  set_msg_config -id {filemgmt 56-12} -suppress
# Suppress the "Unused Sequential Element" warnings introduced in Vivado version 2017.
  set_msg_config -id {Synth 8-6014} -suppress
# Suppress the "Mix of Sync and Async" warnings introduced in Vivado version 2018.
  set_msg_config -string {dma_regin_ext_reg} -suppress
  set_msg_config -string {dma_regin_reg} -suppress
  set_msg_config -string {dma_param_reg} -suppress
  set_msg_config -string {gpio_user_cmd_reg} -suppress
  set_msg_config -string {spi_csb_cmd_reg} -suppress
  set_msg_config -string {synth_cmd_reg} -suppress
# Suppress the "No Routable Loads" warnings on FIFO IP introduced in Vivado version 2018.
  set_msg_config -string {empty_fwft_i} -suppress
  set_msg_config -string {full_fb} -suppress
# Suppress warnings in all model numbers.
  set_msg_config -string {clk_ext} -suppress
  set_msg_config -string {dll_on_rclk} -suppress
  set_msg_config -string {dma_control_reg[5]} -suppress
  set_msg_config -string {dma_control_save} -suppress
  set_msg_config -string {dma_regout_ext} -suppress
  set_msg_config -string {srst_ext} -suppress
  set_msg_config -string {DESC_BRAM} -suppress
  set_msg_config -string {PAGE_BRAM} -suppress
  set_msg_config -string {QDRII_SRAM} -suppress
  set_msg_config -string {XCVR/BPI/CLK200_CLK20/CLKOUT0} -suppress
  set_msg_config -string {XCVR/PCI/EP/U0/inst/pcie_top_i/pcie_7x_i/pcie_bram_top} -suppress
# Suppress model number specific warnings.
switch -exact $modelnumber {
	371 {
		set_msg_config -string {bar0_rddata_reg[127]} -suppress
		set_msg_config -string {clk_measurement} -suppress
		set_msg_config -string {rx_frame_in} -suppress
		set_msg_config -string {tx_clk} -suppress
		set_msg_config -string {tx_srst} -suppress
	}
	372 {
		set_msg_config -string {dma_go_rclk} -suppress
		set_msg_config -string {rx_frame_in} -suppress
	}
	373 {
		set_msg_config -string {bar0_rddata_reg[127]} -suppress
		set_msg_config -string {clk_measurement} -suppress
		set_msg_config -string {rx_frame_in} -suppress
		set_msg_config -string {tx_clk} -suppress
		set_msg_config -string {tx_srst} -suppress
	}
	374 {
		set_msg_config -string {dma_go_rclk} -suppress
		set_msg_config -string {rx_frame_in} -suppress
		set_msg_config -string {rx_clk} -suppress
	}
	376 {
		set_msg_config -string {bar0_rddata_reg[127]} -suppress
		set_msg_config -string {clk_measurement} -suppress
		set_msg_config -string {item_counter} -suppress
		set_msg_config -string {rx_frame_in} -suppress
		set_msg_config -string {sample_count[0]} -suppress
		set_msg_config -string {sample_count[1]} -suppress
		set_msg_config -string {tx_clk} -suppress
		set_msg_config -string {tx_srst} -suppress
	}
	377 {
		set_msg_config -string {clk_measurement} -suppress
		set_msg_config -string {rx_frame_in} -suppress
		set_msg_config -string {tx_clk} -suppress
		set_msg_config -string {tx_srst} -suppress
	}
	378 {
		set_msg_config -string {ADC_ERR} -suppress
		set_msg_config -string {{channel_status} {[1]}} -suppress
		set_msg_config -string {clk_edge_local} -suppress
		set_msg_config -string {dma_regout_ext} -suppress
		set_msg_config -string {dma_status_ext[5]} -suppress
		set_msg_config -string {dma_status_ext[6]} -suppress
		set_msg_config -string {dma_status_ext[7]} -suppress
		set_msg_config -string {clk_measurement} -suppress
		set_msg_config -string {rx_ovr_in} -suppress
		set_msg_config -string {SHARE_ENG.DESC_EXT} -suppress
		set_msg_config -string {tx_clk} -suppress
		set_msg_config -string {tx_srst} -suppress
	}
	default { puts "ERROR: Model number $modelnumber is invalid"; exit }
}
# Suppress SRAM warnings on models not equipped with memory.
if {$sram == "NO"} {
		set_msg_config -string {{error_status} {[6]}} -suppress
		set_msg_config -string {{error_status} {[7]}} -suppress
		set_msg_config -string {qdriip_} -suppress
		set_msg_config -string {sram_} -suppress
		set_msg_config -string {{clock_route} {rst}} -suppress
	}

close_project

puts "INFO: Project SigFPGA_Top created."
