--*******************************************************************************************************************************
--
-- Description: This module acquires data and over-range status from A/D converter(ADC) chips with a serial interface.
--					
-- Rev History: 01-10-2018:	Initial release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all; 

Library UNISIM; 
use UNISIM.Vcomponents.all;

use work.model_specific_package.all;
use work.transceiver_package.all;

entity adc_interface is 
	port (
	--  Reset and clocks supplied by the application logic.
    rx_srst			: in std_logic;		-- Active high synchronous reset in the RX clock domain.
	rx_clk			: in std_logic;		-- Clock used to register ADC sample data.
	
	-- ADC data supplied to the application logic.
	rx_data_out		: out RX_BUS;		-- Data bus carrying ADC samples.
	rx_ovr_out		: out OVR_BIT;		-- Logic one indicates an over-range condition on the ADC.

	-- Serial clock lock command signal supplied from/to the application logic.
	clk_lck_start	: in std_logic; 	-- Initiates the clock lock sequence that sets IODELAY values for data inputs.
	clk_lck_busy	: out std_logic; 	-- Logic one indicates that the clock lock process is active. 
	clk_lck_status	: out std_logic_vector(ADC_CLK_SOURCES-1 downto 0); -- Logic one indicates that clocks and frames are locked.
	
	--	The following signals connect to ADC devices through FPGA pins.
	rx_data_in		: in RX_BUS_IN;		-- ADC data bus.
	rx_frame_in  	: in RX_FRAME_BIT;	-- ADC channel frame sync.
	rx_ovr_in		: in OVR_BIT_IN;	-- ADC over-range flag.
	rx_clk_in		: in RX_CLOCK_BIT	-- ADC output clock.
);
end adc_interface;

architecture behavioral of adc_interface is

	signal adc_fifo_in		: RX_64;			-- ADC samples from all channels within a single device in the divided serial data clock domain.
	signal adc_fifo_out		: RX_64;			-- ADC samples from all channels within a single device in the RX clock domain.
	signal adc_sample		: RX_16;			-- ADC samples from each data ISERDES.
	signal clk_div_clr		: RX_SERDES_1; 		-- Active high clears to the BUFR divider.
	signal clk_div_enable	: RX_SERDES_1; 		-- Active high enables the BUFR divider.
	signal clk_lck_timer	: RX_SERDES_8U; 	-- Timer to delay the start of frame lock until clock lock completes.
	signal data_ce			: RX_SERDES_0;		-- Logic one enables the increment/decrement function in the IDELAY attached to the data bits.
	signal data_clk			: RX_CLOCK_BIT;		-- Serial data clock into the ISERDES.
	signal data_clk_div		: RX_SERDES_1;		-- Divided serial data clock (above).
	signal data_clk_dly		: RX_SERDES_1;		-- Delayed serial data clock (above).
	signal data_clk_rclk	: std_logic;		-- Data clock reclocked in the serial data clock domain.
	signal data_clk_re		: RX_SERDES_0;		-- Rising edge sample of data clock input.
	signal data_div_srst	: RX_SERDES_1;		-- Active high synchronous reset in the divided data clock domain.
	signal data_inc			: RX_SERDES_0;		-- Logic one increments the tap value and logic zero decrements the tap value in the IDELAY attached to the data bits.
	signal data_ld			: RX_SERDES_0;		-- Logic one loads the starting tap value in the IDELAY attached to the data bits.
	signal fifo_rden		: std_logic_vector(1 downto 0); 		-- Active high read enable at the FIFO output.
	signal fifo_ready		: unsigned(ADC_CLK_SOURCES-1 downto 0); -- Bus that conveys the valid bit from all data FIFOs. 
	signal fifo_rst			: RX_SERDES_1; 		-- Active high FIFO asynchronous reset.
	signal fifo_valid		: RX_SERDES_1; 		-- Active high data valid at the FIFO output.
	signal fifo_wren		: RX_SERDES_1; 		-- Active high FIFO write enable.
	signal frame_bitslip	: RX_SERDES_1; 		-- Active high bit slip enable to the ISERDES.
	signal frame_detect		: RX_SERDES_8;		-- Current output byte from the frame ISERDES.
	signal frame_lck_start	: RX_SERDES_1; 		-- Logic one initiates the frame synchronization sequence.
	signal frame_re			: RX_SERDES_1;		-- Rising edge sample of frame input.
	signal fwd_ce			: RX_SERDES_0;		-- Logic one enables the increment/decrement function in the IDELAY attached to the forwarded clock.
	signal fwd_counter		: RX_SERDES_5;		-- Tap count reported by the IDELAY attached to the forwarded clock.
	signal fwd_inc			: RX_SERDES_0;		-- Logic one increments the tap value and logic zero decrements the tap value in the IDELAY attached to the forwarded clock.
	signal fwd_ld			: RX_SERDES_0;		-- Logic one loads the starting tap value in the IDELAY attached to the forwarded clock.
	signal iserdes_rst		: RX_SERDES_1; 		-- Active high resets the ISERDES.
	signal multisync		: std_logic; 		-- Logic one indicates that all devices will synchronize on the next clock.
	signal multisync_bus	: unsigned(ADC_CLK_SOURCES-1 downto 0); -- Bus that conveys the inverted multisync request bit from all frame sync processes. 
	signal multisync_req	: RX_SERDES_1; 		-- Logic one indicates that the ISERDES frame is aligned and waiting for multisync.
	signal multisync_reqn	: RX_SERDES_1; 		-- Logic zero indicates that the ISERDES frame is aligned and waiting for multisync.
	signal rx_data_dly		: RX_BUS_IN;		-- Delayed ADC channel data equal to BUFIO propagation.
	signal rx_frame_dly		: RX_SERDES_1;		-- Delayed ADC channel frame sync equal to BUFIO propagation.
	signal serial_clk		: RX_SERDES_0;		-- Serial data clock at the output of a non-dividing BUFR.
	signal serial_srst		: RX_SERDES_1;		-- Active high synchronous reset in the serial clock domain.
	signal start_rxclk		: std_logic;		-- Clock lock start reclocked in the RX clock domain.
	signal start_serialclk	: RX_SERDES_1;		-- Clock lock start reclocked in the serial clock domain.
	
begin
 	
--************************************************* CREATE DATA CLOCK NETWORK **************************************************

	-- Process each complementary data clock pair forwarded by the ADC.
	CLK_FORWARD : for i in 1 to ADC_CLK_SOURCES generate
	
		-- Set the IDELAY value for the positive data clock using the clock lock controller.
		-- Note: The objective is to set this delay value equal to the propagation through the BUFIO.
		CLKP_IDELAY : IDELAYE2
			generic map(IDELAY_TYPE => "VARIABLE", IDELAY_VALUE => DATA_CLK_INIT(i-1), HIGH_PERFORMANCE_MODE => "TRUE")
			port map(C => serial_clk(i)(0), REGRST => '0', LD => fwd_ld(i)(0), CE => fwd_ce(i)(0), INC => fwd_inc(i)(0), CINVCTRL => '0', CNTVALUEIN => "00000", IDATAIN => rx_clk_in(i+(2*ADC_CLK_SOURCES)), DATAIN => '0', LDPIPEEN => '0', DATAOUT => data_clk_dly(i), CNTVALUEOUT => fwd_counter(i));
		-- Use double data rate (DDR) input registers in the IOB to capture data on the rising and falling edge of the sample clock.
		-- Note: The DDR register is used on SDR inputs to ensure that the signal is registered inside the IOB.
		CLKP_DDR : IDDR
			generic map(DDR_CLK_EDGE => "SAME_EDGE_PIPELINED")
			port map(Q1 => data_clk_re(i)(0), Q2 => open, C => serial_clk(i)(0), CE => '1', D => data_clk_dly(i), R => '0', S => '0');

		-- Assign the complementary data clock to IO clock buffers connected to each frame and data ISERDES.
		CLKP_BUFIO : BUFIO port map (O => data_clk(i), I => rx_clk_in(i));
		CLKN_BUFIO : BUFIO port map (O => data_clk(i+ADC_CLK_SOURCES), I => rx_clk_in(i+ADC_CLK_SOURCES));
		-- Assign the positive data clock to a dividing regional clock buffer connected to each frame and data ISERDES.
		CLKP_DIV_BUFR : BUFR
			generic map(BUFR_DIVIDE => "4")
			port map (O => data_clk_div(i), CE => clk_div_enable(i), CLR => clk_div_clr(i), I => rx_clk_in(i));
		-- Assign the positive data clock to a regional clock buffer used to clear the divided data clock.
		CLKP_BUFR : BUFR port map (O => serial_clk(i)(0), CE => '1', CLR => '0', I => rx_clk_in(i));
		
		-- Create a synchronous reset in the serial clock domain.
		SYNC_SERIALRST : entity work.sync_reset
		port map (
			source_clk		=> rx_clk,
			source_rst		=> rx_srst,
			target_clk  	=> serial_clk(i)(0),
			target_rst		=> serial_srst(i)
		);
	
	end generate;
	
--*************************************************** CLOCK LOCK CONTROLLER ****************************************************

	-- Process each complementary data clock pair forwarded by the ADC.
	CLK_LOCK : for i in 1 to ADC_CLK_SOURCES generate
	
		-- Add delay to the frame and data bits to phase align with the buffered data clocks.
		-- Note: The rising edge of the forwarded clock is only one BUFIO delay behind the data clock, so a rising edge solution is guaranteed within the 2.5 ns search window or the delay is negligible (NOSOL).
		ADC_CLK_LCK : entity work.clock_lock
		generic map (
			SOURCES			=> 1,
			CLK_FWD_INIT	=> DATA_CLK_INIT(i-1),
			MAX_SEARCH		=> ADC_MAX_SEARCH,
			RESET_IF_NOSOL	=> ADC_RESET_IF_NOSOL
		)
		port map (
			srst         	=> serial_srst(i),
			lock_start   	=> clk_lck_start,
			lock_busy   	=> open,
			lock_status    	=> open,
			clk_lck_clk		=> serial_clk(i)(0),
			clk_fwd			=> data_clk_re(i),
			clk_edge_search	=> ADC_CLK_EDGE,
			clk_edge_detect	=> open,
			fwd_counter		=> fwd_counter(i),
			fwd_ld			=> fwd_ld(i),
			fwd_ce			=> fwd_ce(i),
			fwd_inc			=> fwd_inc(i),
			data_ld			=> data_ld(i),
			data_ce			=> data_ce(i),
			data_inc		=> data_inc(i)
		);
		
	end generate;
	
--*************************************************** CREATE FRAME NETWORK *****************************************************

	-- Process each frame signal supplied by the ADC.
	ADC_FRAME : for i in 1 to ADC_CLK_SOURCES generate
	
		-- Set the IDELAY values for the frame using the clock lock controller.
		-- Note: Frame is phase offset 90 degrees from the divided data clock (180 degrees from a data cycle).
		FRAME_IDELAY : IDELAYE2
			generic map(IDELAY_TYPE => "VARIABLE", IDELAY_VALUE => FRAME_INIT(i-1), HIGH_PERFORMANCE_MODE => "TRUE")
			port map(C => serial_clk(i)(0), REGRST => '0', LD => fwd_ld(i)(0), CE => fwd_ce(i)(0), INC => fwd_inc(i)(0), CINVCTRL => '0', CNTVALUEIN => "00000", IDATAIN => rx_frame_in(i), DATAIN => '0', LDPIPEEN => '0', DATAOUT => rx_frame_dly(i), CNTVALUEOUT => open);
		-- Use the SERDES in the IOB to capture the frame with the high speed data clock.
		FRAME_SERDES : ISERDESE2
			generic map(DATA_WIDTH => 8, INTERFACE_TYPE => "NETWORKING", IOBDELAY => "BOTH")
			port map(O => open, Q1 => frame_detect(i)(0), Q2 => frame_detect(i)(1), Q3 => frame_detect(i)(2), Q4 => frame_detect(i)(3), Q5 => frame_detect(i)(4), Q6 => frame_detect(i)(5) , Q7 => frame_detect(i)(6), Q8 => frame_detect(i)(7), SHIFTOUT1 => open, SHIFTOUT2 => open, BITSLIP => frame_bitslip(i) ,CE1 => '1' ,CE2 => '1', CLKDIVP => '0', CLK => data_clk(i), CLKB => data_clk(i+ADC_CLK_SOURCES), CLKDIV => data_clk_div(i), OCLK => '0', DYNCLKDIVSEL => '0', DYNCLKSEL => '0', D => rx_frame_in(i), DDLY => rx_frame_dly(i), OFB => '0', OCLKB => '0', RST => iserdes_rst(i), SHIFTIN1 => '0', SHIFTIN2 => '0');
			
		-- Use double data rate (DDR) input registers in the IOB to capture data on the rising and falling edge of the sample clock.
		-- Note: The DDR register is used on SDR inputs to ensure that the signal is registered inside the IOB.
		FFWD_DDR : IDDR
			generic map(DDR_CLK_EDGE => "SAME_EDGE_PIPELINED")
			port map(Q1 => frame_re(i), Q2 => open, C => serial_clk(i)(0), CE => '1', D => rx_frame_in(i+(ADC_CLK_SOURCES)), R => '0', S => '0');
			
	end generate;

--*************************************************** FRAME SYNC CONTROLLER ****************************************************

	-- Process each frame signal supplied by the ADC.
	FRM_LCK : for i in 1 to ADC_CLK_SOURCES generate
	
		-- Generate a frame lock start command based on a timer.
		process(serial_clk(i)(0))
		begin
			if rising_edge(serial_clk(i)(0)) then
				if serial_srst(i) = '1' then
					clk_lck_timer(i) <= (others => '0');
				else
					-- Reclock the clock lock start input in the serial clock domain.
					start_serialclk(i) <= clk_lck_start;
					-- Start a timer that exceeds the maximum duration possible for a clock lock process.
					-- Note: It is impossible for clock lock to fail in this application, even if an edge is not found.
					if start_serialclk(i) = '1' or clk_lck_timer(i) /= x"00" then
						clk_lck_timer(i) <= clk_lck_timer(i) + x"01";
					else
						clk_lck_timer(i) <= (others => '0');
					end if;
				end if;
			end if;
		end process;
		-- Generate a single period start strobe when the timer expires.
		frame_lck_start(i) <= '1' when clk_lck_timer(i) = x"FF" else '0';
		
		-- Create a synchronous reset in the serial clock domain.
		SYNC_DATARST : entity work.sync_reset
		port map (
			source_clk		=> rx_clk,
			source_rst		=> rx_srst,
			target_clk  	=> data_clk_div(i),
			target_rst		=> data_div_srst(i)
		);
	
		-- Start the frame lock process after the clocks are locked.
		ADC_FRAME_Lock : entity work.frame_lock
		port map (
			data_srst         	=> serial_srst(i),
			data_div_srst       => data_div_srst(i),
			data_clk   			=> serial_clk(i)(0),
			data_clk_div   		=> data_clk_div(i),
			frame_re    		=> frame_re(i),
			lock_start			=> frame_lck_start(i),
			lock_status			=> clk_lck_status(i-1),
			clk_div_clr			=> clk_div_clr(i),
			clk_div_enable		=> clk_div_enable(i),
			iserdes_rst			=> iserdes_rst(i),
			frame_detect		=> frame_detect(i),
			frame_bitslip		=> frame_bitslip(i),
			multisync_req		=> multisync_req(i),
			multisync			=> multisync,
			fifo_rst			=> fifo_rst(i),
			fifo_wren			=> fifo_wren(i)
		);		
	
		-- Invert and reclock all sync request signals to the first serial clock domain.
		process(serial_clk(1)(0))
		begin
			if rising_edge(serial_clk(1)(0)) then
				multisync_reqn(i) <= not(multisync_req(i));
			end if;
		end process;
	
		-- Assign all of the reclocked frame align status bits to a single bus.
		multisync_bus(i-1) <= multisync_reqn(i);
		
	end generate;

	-- Wait for all of the individual clocks to frame align before issuing the command to start forwarding data.
	process(serial_clk(1)(0))
	begin
		if rising_edge(serial_clk(1)(0)) then
			-- Reclock the divided data clock in the serial data clock domain so that it can be sampled.
			data_clk_rclk <= data_clk_div(1);
			-- Wait for all frame alignments to complete and the leading edge of the reference frame to pass.
			if to_integer(multisync_bus) = 0 and data_clk_rclk = '1' then
				multisync <= '1';
			elsif to_integer(multisync_bus) /= 0 then
				multisync <= '0';
			end if;
		end if;
	end process;
	
--**************************************************** CREATE DATA NETWORK *****************************************************

	-- Instantiate the requested number of receiver channels.
	ADC_CHANNEL : for i in 1 to zero_to_one(RX_CHANNEL_COUNT) generate
	
		-- Process each ADC data item bit.
		ADC_DATA : for j in 0 to (RX_ITEM_PER_CLK*RX_ITEM_SIZE/RX_INTERLEAVE)-1 generate
		
			-- Set the IDELAY values for each data bit using the frame lock controller.
			DATA_IDELAY : IDELAYE2
				generic map(IDELAY_TYPE => "VARIABLE", IDELAY_VALUE => ADC_DATA_INIT(((i-1)*RX_ITEM_SIZE/RX_INTERLEAVE)+j), HIGH_PERFORMANCE_MODE => "TRUE")
				port map(C => serial_clk(index(i))(0), REGRST => '0', LD => data_ld(index(i))(0), CE => data_ce(index(i))(0), INC => data_inc(index(i))(0), CINVCTRL => '0', CNTVALUEIN => "00000", IDATAIN => rx_data_in(i)(j), DATAIN => '0', LDPIPEEN => '0', DATAOUT => rx_data_dly(i)(j), CNTVALUEOUT => open);
			-- Use the SERDES in the IOB to capture serialized data with the high speed data clock.
			-- Note: Data is phase offset 90 degrees from the divided data clock (180 degrees from a data cycle).
			DATA_SERDES : ISERDESE2
				generic map(DATA_WIDTH => 8, INTERFACE_TYPE => "NETWORKING", IOBDELAY => "BOTH")
				port map(O => open, Q1 => adc_sample(i)((8*j)+0), Q2 => adc_sample(i)((8*j)+1), Q3 => adc_sample(i)((8*j)+2), Q4 => adc_sample(i)((8*j)+3), Q5 => adc_sample(i)((8*j)+4), Q6 =>adc_sample(i)((8*j)+5) , Q7 => adc_sample(i)((8*j)+6), Q8 => adc_sample(i)((8*j)+7), SHIFTOUT1 => open, SHIFTOUT2 => open, BITSLIP => frame_bitslip(index(i)), CE1 => '1' ,CE2 => '1', CLKDIVP => '0', CLK => data_clk(index(i)), CLKB => data_clk(index(i)+ADC_CLK_SOURCES), CLKDIV => data_clk_div(index(i)), OCLK => '0', DYNCLKDIVSEL => '0', DYNCLKSEL => '0', D => '0', DDLY => rx_data_dly(i)(j), OFB => '0', OCLKB => '0', RST => iserdes_rst(index(i)), SHIFTIN1 => '0', SHIFTIN2 => '0');

		end generate;

		-- Tie the over-range bit low since there is none available from the ADC.
		rx_ovr_out(i) <= "0";
		
	end generate;
	
--************************************************* RX CLOCK DOMAIN PROCESSING *************************************************

	-- Use a FIFO to cross from the data clock domain to the RX clock domain.
	DATA_XING : for i in 1 to ADC_CLK_SOURCES generate
	
		-- Consolidate the individual data busses assigned to a single device into a single data bus that feeds the FIFO.
		DATA_IN : for j in 0 to RX_FRAME_SHARE-1 generate
			adc_fifo_in(i)((j*RX_ITEM_SIZE)+RX_ITEM_SIZE-1 downto j*RX_ITEM_SIZE) <= adc_sample(((i-1)*RX_FRAME_SHARE)+j+1);
		end generate;

		-- Note: RX clock must be set equal to or greater than the fastest data clock.
		ADC_FIFO : entity work.fifo_16x64
		port map (
			rst         => fifo_rst(i),
			wr_clk   	=> data_clk_div(i),
			rd_clk   	=> rx_clk,
			din    		=> adc_fifo_in(i),
			wr_en		=> fifo_wren(i),
			rd_en		=> fifo_rden(0),
			dout		=> adc_fifo_out(i),
			full		=> open,
			empty		=> open,
			valid		=> fifo_valid(i)
		);
		-- Assign inverted FIFO valid bits to a single bus.
		fifo_ready(i-1) <= not(fifo_valid(i));

		-- Subdivide the FIFO output across the output data ports.
		DATA_OUT : for j in 0 to RX_FRAME_SHARE-1 generate
			rx_data_out(((i-1)*RX_FRAME_SHARE)+j+1) <= adc_fifo_out(i)((j*RX_ITEM_SIZE)+RX_ITEM_SIZE-1 downto j*RX_ITEM_SIZE);
		end generate;
		
	end generate;
		
	-- Create a busy bit that spans the start of clock lock to valid data available at the FIFO output.
	process(rx_clk)
	begin
		if rising_edge(rx_clk) then
			if rx_srst = '1' then
				clk_lck_busy <= '0';
			else
				-- Synchronize data in the RX clock domain by waiting for all of the FIFO outputs to align.
				if to_integer(fifo_ready) = 0 then
					fifo_rden(0) <= '1';
				else
					fifo_rden(0) <= '0';
				end if;
				-- Register the FIFO read enable so that it can be edge detected.
				fifo_rden(1) <= fifo_rden(0);
				-- Reclock the clock lock start input in the RX clock domain.
				start_rxclk <= clk_lck_start;
				-- Assert the busy bit when clock lock begins and deassert when frame sync concludes.
				if start_rxclk = '1' then
					clk_lck_busy <= '1';
				elsif fifo_rden(1) <= '0' and fifo_rden(0) <= '1' then
					clk_lck_busy <= '0';
				end if;
			end if;
		end if;
	end process;
	
end behavioral;
